﻿using System.Diagnostics;
using System.IO;
using System.Linq;
using Dss.Api.Examples.Internal;
using System;
using DataScope.Select.Api;
using DataScope.Select.Api.EvaluationConnect;
using DataScope.Select.Api.Content;

namespace Dss.Api.Examples.EvaluationConnect
{

    [Example("Evaluation Connect")]
    public class ChallengeBatchExamples
    {

        #region Create

        [Example("Validate a Bond Identifier")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void ValidateIdentifier()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "ValidateBond", MethodType.Operation, Publish.Primary);
            //Validate the bond.  The validated bond can be used in place of the InstrumentIdentifier 
            //when creating challenges.  This will ensure the instrument is a valid bond priced by LSEG Pricing Service before submitting the
            //challenge.
            var validatedBond = EvaluationConnectContext.ChallengeOperations.ValidateBond("001814AS1=RRPS", IdentifierType.Ric);

            //Output
            Status.WriteLine("Identifier: " + validatedBond.Identifier);
            Status.WriteLine("Identifier Type: " + validatedBond.IdentifierType);
            Status.WriteLine("Description: " + validatedBond.Description);
            Status.WriteLine("Instrument Type: " + validatedBond.InstrumentType);
            Status.WriteLine("Key: " + validatedBond.Key);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Validate a Bond Identifier: Invalid")]
        public void ValidateIdentifierInvalid()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "ValidateBond", MethodType.Operation, Publish.Secondary);
            try
            {
                //Try to validate in invalid ric
                EvaluationConnectContext.ChallengeOperations.ValidateBond("BADRIC=RRPS", IdentifierType.Ric);
            }
            catch (ValidationException ex)
            {
                Status.WriteLine(ex.Message);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Confirm Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateConfirmChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Primary);
            //Create the confirm challenge
            EvaluationConnectContext.ChallengeOperations.CreateConfirmChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt, 
                "Please confirm that the price is correct.");
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Pre-Validated Bond")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreatePreValidatedBond()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "ValidateBond");
            //Validate the bond.  The validated bond can be used in place of the InstrumentIdentifier 
            //when creating challenges.  This will ensure the instrument is a valid bond priced by LSEG Pricing Service before submitting the
            //challenge.
            var validatedBond = EvaluationConnectContext.ChallengeOperations.ValidateBond("001814AS1=RRPS", IdentifierType.Ric);
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Primary);
            //Create the confirm challenge
            EvaluationConnectContext.ChallengeOperations.CreateConfirmChallenge(
                validatedBond,
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.");
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Bad Identifier")]
        public void CreateBadIdentifier()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            try
            {
                //Create the confirm challenge
                EvaluationConnectContext.ChallengeOperations.CreateConfirmChallenge(
                    InstrumentIdentifier.Create(IdentifierType.Ric, "BAD_RIC"),
                    DateTime.Today,
                    PricingRun.Asia18Jst,
                    "Please confirm that the price is correct.");
            }
            catch (ValidationException ex)
            {
                Status.WriteLine(ex.Message);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Unchanged Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateUnchangedChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the unchanged challenge
            EvaluationConnectContext.ChallengeOperations.CreateUnchangedChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.");
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Tolerance Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateToleranceChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the tolerance challenge
            EvaluationConnectContext.ChallengeOperations.CreateToleranceChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.");
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Trade Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateTradeChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the trade challenge
            var broker = BrokerInfo.Create(
                "Broker's Name", "Broker's Firm", "name@firm.com", "Phone", "Note");
            EvaluationConnectContext.ChallengeOperations.CreateTradeChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.",
                broker,
                PricePoint.Create(23.52M, DateTime.Today),
                PriceType.Ask, 
                40123); //Volume
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Vendor Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateVendorChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the vendor challenge
            EvaluationConnectContext.ChallengeOperations.CreateVendorChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.",
                "Vendor Name",
                PricePoint.Create(23.52M, DateTime.Today),
                PriceType.Ask);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Two Vendor Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateVendorChallengeTwoVendors()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the vendor challenge
            var challengeRequest = ChallengeRequest.CreateVendor(InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.",
                "Vendor Name",
                PricePoint.Create(23.52M, DateTime.Today),
                PriceType.Ask,
                vendorName2: "Second Vendor",
                vendorPrice2: PricePoint.Create(23.50M, DateTime.Today),
                priceType2: PriceType.Ask);
            EvaluationConnectContext.ChallengeOperations.Create(challengeRequest);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Three Vendor Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateVendorChallengeThreeVendors()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the vendor challenge
            var challengeRequest = ChallengeRequest.CreateVendor(InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.EndOfDay,
                "Please confirm that the price is correct.",
                "Vendor Name",
                PricePoint.Create(23.52M, DateTime.Today),
                PriceType.Ask,
                vendorName2: "Second Vendor",
                vendorPrice2: PricePoint.Create(23.50M, DateTime.Today),
                priceType2: PriceType.Ask,
                vendorName3: "Third Vendor",
                vendorPrice3: PricePoint.Create(23.53M, DateTime.Today),
                priceType3: PriceType.Ask);
            EvaluationConnectContext.ChallengeOperations.Create(challengeRequest);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Create: Broker Challenge")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void CreateBrokerChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Create the broker challenge
            var broker = BrokerInfo.Create(
                "Broker's Name", "Broker's Firm", "name@firm.com", "Phone", "Note");
            EvaluationConnectContext.ChallengeOperations.CreateBrokerChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), //Also accepts validated bonds
                DateTime.Today,
                PricingRun.Amers15EstEdt,
                "Please confirm that the price is correct.",
                broker,
                23.52M,
                PriceType.Ask);
            Status.EndNotify(EvaluationConnectContext);
        }


        [Example("Delete")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void DeleteChallenge()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Primary);
            //Create challenge
            var askTrpsChallenge = EvaluationConnectContext.ChallengeOperations.CreateToleranceChallenge(InstrumentIdentifier.Create(IdentifierType.Ric, "009158AJ5=RRPS"), DateTime.Today,
                PricingRun.Amers15EstEdt, "Please confirm that the price is correct");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "Delete", MethodType.Operation, Publish.Primary);
            //Delete the challenge.  
            EvaluationConnectContext.ChallengeOperations.Delete(askTrpsChallenge.ChallengeId);
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

        #region Attachments

        [Example("Create: With Attachments")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(ChallengeAttachment))]
        public void CreateChallengeWithAttachments()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Create", MethodType.Operation, Publish.Secondary);
            //Load the attachment contents from a file
            var contents = File.ReadAllBytes(@"..\..\AskTrps\Resources\AttachmentFile.txt");

            //Create the challenge (with the attachment)
            //See also Get Challenge.Attachments and Download Attachment Contents examples
            var challenge = EvaluationConnectContext.ChallengeOperations.CreateConfirmChallenge(
                InstrumentIdentifier.Create(IdentifierType.Ric, "001814AS1=RRPS"), 
                DateTime.Today, PricingRun.EndOfDay, "test", 
                new [] { ChallengeRequestAttachment.Create("AttachmentFile.txt", contents) });
           // Status.EndNotify(EvaluationConnectContext);

            //Output
            Status.WriteLine(String.Join(", ", challenge.Attachments.Select(a => a.Name)));
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Lazy Load: Challenge.Attachments")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(ChallengeAttachment))]
        public void GetChallengeAttachments()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get");
            //Fetch the challenge
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAA");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "ChallengeAttachment", MethodType.Association, Publish.Primary);
            //Get the list of attachments for this challenge.  See also Downloading Attachment Contents example.
            //Attachments navigation property is null until loaded from the server.
            EvaluationConnectContext.LoadProperty(challenge, c => c.Attachments);

            //Output
            Status.WriteLine(String.Join(", ", challenge.Attachments.Select(a=>a.Name)));
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Download Attachment Contents")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(ChallengeAttachment))]
        public void DownloadAttachmentContents()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get");
            //Fetch the challenge
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAA");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "Attachments");
            //Get the list of attachments for this challenge.  See also Downloading Attachment Contents example.
            EvaluationConnectContext.LoadProperty(challenge, c => c.Attachments);
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "ChallengeAttachment", "GetDefaultStream", MethodType.Stream, Publish.Primary);
            var firstAttachment = challenge.Attachments.First();

            //Download the contents for this stream
            var responseStream = EvaluationConnectContext.ChallengeAttachmentOperations.GetReadStream(firstAttachment);
            using (var reader = new StreamReader(responseStream.Stream))
                Status.WriteLine(reader.ReadToEnd());
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

        #region Query

        [Example("Get a Challenge By Id")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void GetChallengeById()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get", MethodType.Operation, Publish.Primary);
            //Fetch the challenge
            var retrievedChallenge = EvaluationConnectContext.ChallengeOperations.Get("999AAD");

            //Output
            Status.WriteLine(retrievedChallenge.ValidatedBond.Description);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Get a Challenge By Id (Not Found)")]
        public void GetChallengeById_NotFound()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            try
            {
                Status.Notify(EvaluationConnectContext, "Challenge", "Get");
                //Fetch the challenge
                EvaluationConnectContext.ChallengeOperations.Get("Bad Id");
            }
            catch (NotFoundException)
            {
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Get a Challenge By Id With Bond Price Recipe")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(BondPriceRecipe))]
        public void GetChallengeById_WithBondPriceRecipe()
        {
            // Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "BondPriceRecipe", "Get", MethodType.Operation, Publish.Secondary);

            // Fetch the challenge. Replace "999AAD" with desired Challenge ID.
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAD", c => c.BondPriceRecipe);

            // Output
            Status.WriteLine(challenge.BondPriceRecipe.BenchmarkType);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Get My Active Challenges")]
        public void GetActiveChallenges()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

			Status.Notify(EvaluationConnectContext, "Challenge", "GetMyActive", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var activeChallenges = EvaluationConnectContext.ChallengeOperations.GetMyActive();

            //Display the the descriptions of all challenged bonds
            foreach (var challenge in activeChallenges)
                Status.WriteLine(challenge.ValidatedBond.Description);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Get My Resolved Challenges for Today")]
        public void GetResolvedChallengesForToday()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

			Status.Notify(EvaluationConnectContext, "Challenge", "GetMyResolved", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var resolvedChallenges = EvaluationConnectContext.ChallengeOperations.GetMyResolved(
                new DateTime(2015, 1, 20), DateTime.Today);

            //Display the the descriptions of all challenged bonds
            foreach (var challenge in resolvedChallenges)
                Status.WriteLine(challenge.ValidatedBond.Description);
            Status.EndNotify(EvaluationConnectContext);

        }

        [Example("Get Company's Challenges")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void GetCompanysChallenges()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetCompany", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var resolvedChallenges = EvaluationConnectContext.ChallengeOperations.GetCompany(
                new DateTime(2015, 1, 20), DateTime.Today);

            //Display the the descriptions of all challenged bonds
            foreach (var challenge in resolvedChallenges)
                Status.WriteLine(challenge.ValidatedBond.Description);
            Status.EndNotify(EvaluationConnectContext);

        }

        [Example("Get All Challenges")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        public void GetAllChallenges()
        {
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetAll", MethodType.Operation, Publish.Primary);
            var results = EvaluationConnectContext.ChallengeOperations.GetAll();

            //Output
            foreach (var challenge in results)
                Status.WriteLine(challenge.ChallengeId);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Get All Challenges With Bond Price Recipe")]
        [EndToEndExample(typeof(EvaluationConnectContext))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(BondPriceRecipe))]
        public void GetAllChallengesWithBondPriceRecipe()
        {
            // Create context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "BondPriceRecipe", "GetAll", MethodType.Operation, Publish.Secondary);
            // Fetch all challenges and the bond price recipe for each challenge.
            var results = EvaluationConnectContext.ChallengeOperations.GetAll(c => c.BondPriceRecipe);

            //Output
            foreach (var challenge in results)
            {
                Status.WriteLine(challenge.BondPriceRecipe.PricingMethodCode);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

        #region History

        [Example("Get Challenge History")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(ChallengeHistoryItem))]
        public void GetChallengeHistory()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get");
            //Fetch the challenge.
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAF");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "ChallengeHistoryItem", MethodType.Association, Publish.Primary);
            //Fetch the history for this challenge.
            EvaluationConnectContext.LoadProperty(challenge, c=>c.History);

            //Display some information
            foreach (var history in challenge.History)
                Status.WriteLine("{0}: {1}", history.ChallengeVersion, history.Description);
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

        #region Comments

        [Example("Add Comment")]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(Challenge))]
        [EndToEndExample(typeof(EvaluationConnectContext), typeof(ChallengeHistoryItem))]
        public void AddComment()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get");
            //Fetch the challenge.
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAF");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "AddComment", MethodType.Operation, Publish.Primary);
            //Add a note to the evaluator
            EvaluationConnectContext.ChallengeOperations.AddComment(challenge, "Please call me at 213-456-1121.  Thanks!");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "History");
            //Retrieve all comments for this challenge
            EvaluationConnectContext.LoadProperty(challenge, c => c.History);

            //Display some information
            foreach (var history in challenge.History)
                Status.WriteLine("{0}: {1}", history.ChallengeVersion, history.Description);
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Add Comment (Error)")]
        public void AddCommentError()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "Get");
            //Fetch the challenge.
            var challenge = EvaluationConnectContext.ChallengeOperations.Get("999AAF");
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "AddComment", MethodType.Operation, Publish.Secondary);
            //Add a note to the evaluator
            EvaluationConnectContext.ChallengeOperations.AddComment(challenge, ""); //Error: Empty comment
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

        #region Reports

        [Example("Reports: Get Monthly Challenge Report")]
        public void GetMonthlyReport()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetMonthlyReport", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var challengeReport = EvaluationConnectContext.ChallengeOperations.GetMonthlyReport(2014, 10, 2015, 2, true);

            //Display the the descriptions of all challenged bonds
            foreach (var record in challengeReport.Records)
            {
                Status.WriteLine("Records changed: {0}", record.Changed);
                Status.WriteLine("Records not changed: {0}", record.NotChanged);
                Status.WriteLine("Total records", record.Total);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Reports: Get Sector Challenge Report")]
        public void GetSectorReport()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetSectorReport", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var challengeReport = EvaluationConnectContext.ChallengeOperations.GetSectorReport(2014, 3, 2015, 2, true);

            //Display the the descriptions of all challenged bonds
            foreach (var record in challengeReport.Records)
            {
                Status.WriteLine("Records changed up: {0}", record.ChangedUp);
                Status.WriteLine("Records changed down: {0}", record.ChangedDown);
                Status.WriteLine("Total records", record.Total);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Reports: Get Evaluator Monthly Challenge Report")]
        public void GetEvaluatorMonthlyReport()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetEvaluatorMonthlyReport", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var challengeReport = EvaluationConnectContext.ChallengeOperations.GetEvaluatorMonthlyReport(2014, 10, 2015, 2, 65598);

            //Display the the descriptions of all challenged bonds
            foreach (var record in challengeReport.Records)
            {
                Status.WriteLine("Records changed: {0}", record.Changed);
                Status.WriteLine("Records not changed: {0}", record.NotChanged);
                Status.WriteLine("Total records", record.Total);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Reports: Get Evaluator Sector Challenge Report")]
        public void GetEvaluatorSectorReport()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetEvaluatorSectorReport", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var challengeReport = EvaluationConnectContext.ChallengeOperations.GetEvaluatorSectorReport(2014, 3, 2015, 2, 65598);

            //Display the the descriptions of all challenged bonds
            foreach (var record in challengeReport.Records)
            {
                Status.WriteLine("Records changed up: {0}", record.ChangedUp);
                Status.WriteLine("Records changed down: {0}", record.ChangedDown);
                Status.WriteLine("Total records", record.Total);
            }
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Reports: Get Todays Challenge Infos for Bond")]
        public void GetTodaysChallengeInfosForBond()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetTodaysChallengeInfosForBond", MethodType.Operation, Publish.Primary);
            //Fetch all active challenges
            var challengeReport = EvaluationConnectContext.ChallengeOperations.GetTodaysChallengeInfosForBond("0x0000fa0004c9aa9d", "0x00040500013d3c01");

            //Display the the descriptions of all challenged bonds
            foreach (var record in challengeReport)
            {
                Status.WriteLine("Challenge Type: ", record.Type);
                Status.WriteLine("Challenge Valuation: ", record.Valuation);
                Status.WriteLine("Challenge Outcome: ",record.Outcome);
                Status.WriteLine("Challenge EvaluationTime: ", record.EvaluationTime);
            }
            Status.EndNotify(EvaluationConnectContext);
        }
        #endregion

        #region Export

        [Example("Export: Get list of exportable fields")]
        public void GetExportableFields()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetExportableFields", MethodType.Operation, Publish.Primary);
            //Fetch the list of export available fields
            var fields = EvaluationConnectContext.ChallengeOperations.GetExportableFields();

            //Output
            Status.WriteLine(String.Join(", ", fields.Select(f=>f.FieldName)));
            Status.EndNotify(EvaluationConnectContext);
        }

        [Example("Export")]
        public void Export()
        {
            //Create the context
            var EvaluationConnectContext = ContextHelper.CreateEvaluationConnectContext();

            Status.Notify(EvaluationConnectContext, "Challenge", "GetExportableFields");
            //Fetch the list of export available fields
            //For this example we will export all the available fields
            var fields = EvaluationConnectContext.ChallengeOperations.GetExportableFields()
                .ToArray()
                .Select(f=>f.FieldName);
            Status.EndNotify(EvaluationConnectContext);

            Status.Notify(EvaluationConnectContext, "Challenge", "Export", MethodType.Operation, Publish.Primary);
            //Specify the challenge id's to export
            var ids = new[] {"999AAA", "999AAB", "999AAC", "999AAD"};
            var export = EvaluationConnectContext.ChallengeOperations.Export(ids, fields);

            //Output
            foreach (var row in export.Split('\n').Select(s=>s.TrimEnd('\r')))
                Status.WriteLine(row);
            Status.EndNotify(EvaluationConnectContext);
        }

        #endregion

    }

}
