﻿using Dss.Api.Examples.Properties;
using System;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Dss.Api.Examples.Internal
{
    public class CodeSnippet
    {
        private static readonly Regex NewSuffixRegex = new Regex(@"\s*\+\s*Suffix.New\(\)", RegexOptions.Singleline);
        private static readonly Regex NewContextRegex = new Regex(@"new [a-zA-Z]{1,32}Context\(", RegexOptions.Singleline);

        public int PrependendLineCount { get; private set; }
        public string Code { get; private set; }
        public int? StartLine { get; private set; }
        public int? EndLine { get; private set; }
        private readonly CodeSettings _settings;
        public string Description { get; private set; }

        public CodeSnippet(CodeSettings settings, string code, int? startLine = null, int? endLine = null)
        {
            _settings = settings;
            Code = code;
            StartLine = startLine;
            EndLine = endLine;
            Description = ExtractDescription(Code);
        }

        public string CleanCode(bool addContext, bool removeDescription)
        {
            if (Code == null)
                return Code;
            var lines = Code.Split('\n');
            if (lines.Count() == 1)
                return Code.TrimStart(' ', '\t');
            //Determine how many whitespace characters should be removed from the left
            var charsToStrip = lines
                .Where(line => !String.IsNullOrWhiteSpace(line))
                .Select(CountSpaceChars)
                .Where(cnt => cnt >= 0)
                .DefaultIfEmpty()
                .Min(count => count);
            //Clean up the code
            var inComment = false;
            var cleanLines = lines
                .Where(line => ShouldIncludeLine(line, removeDescription, ref inComment))
                .Select(line => line.Length <= charsToStrip ? "" : line.Substring(charsToStrip))
                .Select(line => NewSuffixRegex.Replace(line, "")) //Remove the suffix generator
                .Select(line => line.Replace("Status.WriteLine", "Debug.WriteLine"))
                .Select(line => line.Replace("streamContents = ", ""))
                .Select(line => _settings.SanitizeUrls ? line.Replace("Settings.Default.ServiceUri", "\"" + _settings.ProductionUrl + "\"") : line)
                .Select(line => line.Replace("Settings.Default.UserId", "\"<your user id>\""))
                .Select(line => line.Replace("Settings.Default.Password", "\"<your password>\""))
                .Select(line => line.Replace("ContextHelper.ClearToken();", ""))
	            .Select(line => line.Replace("string internalStreamOutput;", ""))
	            .Select(line => line.Replace("internalStreamOutput = ",  ""))
                .Select(FixCreateContext);

            //Create Context
            if (!Code.Contains("ContextHelper.Create") && !NewContextRegex.IsMatch(Code) && addContext)
            {
                PrependendLineCount++;
                cleanLines = new[] { CreateContextFrom(Code) }.Concat(cleanLines);
            }

            //Done
            return String.Join("\n", cleanLines);
        }

        internal static bool ShouldIncludeLine(string line, bool removeDescription, ref bool inComment)
        {
            if (line.Contains("Status.Notify("))
                return false;
            if (line.Contains("Status.EndNotify("))
                return false;
            if (line.Contains("Cleanup();"))
                return false;
            if (line.Contains("string streamContents"))
                return false;
            if (line.Contains("Debug.Write"))
                return false;
            if (line.Contains("context.Preferences.WaitSeconds"))
                return false;
            if (line.Contains(".ActiveMocks()"))
                return false;
            if (line.Contains("TODO:"))
                return false;
            //Multi-line comments (description)
            if (removeDescription)
            {
                if (line.TrimStart().StartsWith("/*") && !inComment)
                {
                    inComment = true;
                    return false;
                }
                if (line.TrimEnd().EndsWith("*/") && inComment)
                {
                    inComment = false;
                    return false;
                }
                if (inComment)
                    return false;
            }
            return true;
        }

        private string FixCreateContext(string line)
        {
            if (!line.Contains("ContextHelper.Create"))
                return line;
            line = line.Replace("ContextHelper.Create", "");
            line = line.Replace("true", "");		// blank out value of optional parameter true
            line = line.Replace("false", "");		// blank out value of optional parameter false
            var assignment = line.Split('=');
            if (assignment.Length != 2)
                return line;

            var context = assignment[1].TrimEnd('(', ')', ' ', ';', '\r', '\n');
            return CreateContext(context, assignment[0]);
        }

        private string CreateContextFrom(string snippet)
        {
            var regex = new Regex("(?<context>[A-Za-z]{1,20}Context)\\.", RegexOptions.Singleline);
            var matches = regex.Matches(snippet);
            if (matches.Count == 0)
                return String.Empty;
            var context = matches[0].Groups["context"].Value;
            return CreateContext(context, String.Format("var {0} ", context));
        }

        private string CreateContext(string name, string assignment = null)
        {
            const string username = "<your user id>";
            const string password = "<your password>";
            var url = _settings.SanitizeUrls ? _settings.ProductionUrl : Settings.Default.ServiceUri;
            return String.Format("{0}= new {1}(new Uri(\"{2}\"), \"{3}\", \"{4}\");", assignment, name, url, username, password);
        }

        private static int CountSpaceChars(string line)
        {
            if (String.IsNullOrEmpty(line))
                return -1;
            var index = 0;
            var spaceChar = line[0];
            if (spaceChar != ' ' && spaceChar != '\t')
                return 0;
            while (index < line.Length && (line[index++] == spaceChar))
                ;
            return --index;
        }

        private static string ExtractDescription(string code)
        {
            if (code == null)
                return null;
            var description = new StringBuilder();
            var lines = code.Split('\n');
            var inComment = false;
            foreach (var line in lines)
            {
                var commentLine = line;
                if (string.IsNullOrWhiteSpace(line))
                    continue;
                if (commentLine.TrimStart().StartsWith("/*") && !inComment)
                {
                    inComment = true;
                    commentLine = commentLine.TrimStart().Substring(3);
                }
                if (commentLine.TrimEnd().EndsWith("*/") && inComment)
                {
                    commentLine = commentLine.Trim().Replace("*/", "");
                    if (commentLine.StartsWith("*"))
                        commentLine = commentLine.Substring(2);

                    description.AppendLine(commentLine + "\n");

                    if (String.IsNullOrWhiteSpace(commentLine))
                        commentLine = null;
                    inComment = false;
                }
                if (inComment && commentLine != null)
                {
                    if (commentLine.TrimStart().StartsWith("*"))
                        commentLine = commentLine.TrimStart().Substring(2);
                    description.AppendLine(commentLine + "\n");
                }
            }
            return description.ToString();
        }
    }
}
