﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;

namespace Dss.Api.Examples.Internal
{
    public class ExampleStep
    {
        public Example Example { get; private set; }
        public string StepNumber { get; set; }
        public string ContextName { get; set; }
        public string EntityName { get; set; }
        public string MethodName { get; set; }
        public ExampleStepResult Result { get; set; }
        public MethodType MethodType { get; set; }
        public Publish Publish { get; set; }

        public string ElapsedTime
        {
            get
            {
                return (Result == null) ? "" : Result.ElapsedTime;
            }
        }

        public string ResponseTime
        {
            get
            {
                return (Result == null) ? "" : Result.ResponseTime;
            }
        }

        public string StatusCode
        {
            get
            {
                return (Result == null) ? "" : (Result.StatusCode).ToString(CultureInfo.InvariantCulture);
            }
        }

        public string Verb
        {
            get
            {
                return (Result == null) ? "" : Result.Verb.ToString();
            }
        }

        public string Description
        {
            get
            {
                return string.Format("{0}.{1}", EntityName, MethodName);
            }
        }

        public bool IsPrimary { get { return Publish == Publish.Primary; } }

        public static ExampleStep Create(
            Example example,
            ResponseCapture responseCapture,
            Exception error,
            StackTrace stackTrace,
            string elapsedTime,
            string stepNumber,
            string contextName,
            string entityName,
            string methodName,
            MethodType methodType,
            Publish publish,
            string output,
            string responseBody)
        {
            //Create the event
            var step = new ExampleStep
            {
                Example = example,
                StepNumber = stepNumber,
                ContextName = contextName,
                EntityName = entityName,
                MethodName = methodName,
                MethodType = methodType,
                Publish = publish,
            };
            //Request details
            var args = responseCapture.EventArgs;
            if (args != null && args.Request != null)
            {
                step.Result = new ExampleStepResult();
                step.Result.Status = (args.Response == null) ? HttpStatusCode.Unused : args.Response.StatusCode;
                step.Result.ElapsedTime = elapsedTime;
                step.Result.ResponseTime = args.ElapsedTime.ToString();
                step.Result.Output = output;
                step.Result.Error = error;
                step.Result.Uri = args.Request.RequestUri;
                step.Result.Verb = args.Request.Method;

                var headers = args.Request.Headers.ToDictionary(header => header.Key, header => string.Join(";", header.Value));
                if (args.Request.Content != null && args.Request.Content.Headers != null)
                {
                    foreach (var header in args.Request.Content.Headers.Where(header => !headers.ContainsKey(header.Key)))
                    {
                        headers.Add(header.Key, string.Join(";", header.Value));
                    }
                }
                step.Result.RequestHeaders = headers;
                step.Result.RequestBody = args.Request.Content == null ? null : args.Request.Content.ReadAsStringAsync().Result;

            }
            //Response details
            if (args != null && args.Response != null)
            {
                step.Result = step.Result ?? new ExampleStepResult();
                step.Result.ResponseHeaders = args.Response.Headers.ToDictionary(header => header.Key, header => header.Value.First());
                if ( responseBody != null ) {
                    step.Result.ResponseBody = responseBody;
                } else {
                    try {
                        step.Result.ResponseBody = args.Response.Content == null ? null : args.Response.Content.ReadAsStringAsync().Result;
                    } catch (Exception ex) {
                        step.Result.ResponseBody = string.Format("Error reading content:\r\n\r\n{0} \r\n\r\nIf response content is transfer encoded as chucked (content is read by a stream), then the example's call to Status.EndNotify needs to include the response body as a parameter to the method.", ex.GetBaseException().Message);
                    }
                }
            }
            //Return
            return step;
        }

        public static bool IsAscii(string value)
        {
            // ASCII encoding replaces non-ascii with question marks, so we use UTF8 to see if multi-byte sequences are there
            return Encoding.UTF8.GetByteCount(value) == value.Length;
        }

        public override string ToString()
        {
            return string.Format("{0}.{1}:: {2}{3}.{4} ({5} {6})",
                Example.Name.Split('.').Last(),
                StepNumber,
                ContextName,
                EntityName == null ? "" : "." + EntityName,
                MethodName,
                Result == null ? 0 : (int)Result.Status, 
                Result == null ? "" : Result.Status.ToString());
        }
    }
}
