﻿using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Dss.Api.Examples.Internal;
using Dss.Api.Examples.Properties;
using DataScope.Select.Core.RestApi.Helpers;

namespace Dss.Api.Examples
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private int _lastTabIndex;
        private bool _changingTabs;

        public MainWindow()
        {
            InitializeComponent();

            if (Settings.Default.UserId == "YourUserId")
            {
                var form = new Login();
                form.ShowDialog();
                if (form.DialogResult.HasValue && form.DialogResult.Value)
                {
                    Settings.Default.UserId = form.UserName.Text;
                    Settings.Default.Password = form.Password.Password;
                }
                else
                    Close();

                form.Close();
            }

            SwitchMode();

            ErrorBorder.Visibility = Visibility.Collapsed;

#if !DEBUG
            Export.Visibility = Visibility.Collapsed;
#endif

            //Load the examples
            LoadNavigator();

            //Setup syntax highlighter
            StatusDetailsResponse.FontFamily = new FontFamily("Consolas");
            StatusDetailsRequest.FontFamily = new FontFamily("Consolas");
            StatusDetailsCSharp.FontFamily = new FontFamily("Consolas");
            StatusDetailsCSharpFull.FontFamily = new FontFamily("Consolas");

            VersionLabel.Text = BuildStatusBarText();

            ExportProgress.Visibility = Visibility.Hidden;
            ExportProgressText.Visibility = Visibility.Hidden;
            ExportExampleName.Visibility = Visibility.Hidden;
            RedImage.Visibility = Visibility.Hidden;
            FailedCount.Visibility = Visibility.Hidden;
        }

        private static string BuildStatusBarText()
        {
            return string.Format("Copyright @{0} London Stock Exchange Group plc and its group of companies (LSEG) and/or its affiliates. All rights reserved", DateTime.Now.ToString("yyyy"));
        }

        private void StatusList_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            //Get the status information
            var enumerator = e.AddedItems.GetEnumerator();
            ExampleStep step;
            if (!enumerator.MoveNext() || (step = enumerator.Current as ExampleStep) == null)
            {
                if (Status.CurrentExample == null || !Status.CurrentExample.Steps.Any())
                {
                    StatusDetailsRequest.Text = "";
                    StatusDetailsResponse.Text = "";
                    StatusDetailsCSharp.Text = "";
                }
                return;
            }

            var truncateResults = false;
            if (TruncateResultsCheck.IsChecked.HasValue)
                truncateResults = TruncateResultsCheck.IsChecked.Value;

            //Update the display
            var extractor = new ExampleExtractor(ExportProgress, ExportProgressText, RedImage, FailedCount, false, truncateResults);
            StatusDetailsRequest.FontFamily = new FontFamily("Consolas");

            //HTTP Request & Response
            StatusDetailsRequest.Text = extractor.GetHttpRequestSnippet(step);
            StatusDetailsResponse.Text = extractor.GetHttpResponseSnippet(step);

            //C# Code
            var fullCode = extractor.GetCSharpFull(step.Example);

            //C# Full
            StatusDetailsCSharpFull.Text = fullCode.CleanCode(true, true);
            StatusDetailsCSharpFull.TextArea.TextView.BackgroundRenderers.Clear();

            //C# Snippet
            var snippet = extractor.GetCSharpSnippet(fullCode, step);
            StatusDetailsCSharp.Text = snippet == null ? "" : snippet.Code;
            if (snippet != null && Status.CurrentExample != null && Status.CurrentExample.Steps != null && Status.CurrentExample.Steps.Count() > 1)
                StatusDetailsCSharpFull.TextArea.TextView.BackgroundRenderers.Add(
                    new DiffLineBackgroundRenderer(snippet.StartLine, snippet.EndLine));

            //Error
            if (step.Result != null && step.Result.HasError)
            {
                ErrorBorder.Visibility = Visibility.Visible;
                ErrorMessage.Text = step.Result.ErrorMessage;
                var ex = step.Result.Error;
                var details = new StringBuilder();
                while (ex != null)
                {
                    details.Append(ex);
                    ex = ex.InnerException;
                }
                ErrorDetails.Text = details.ToString();
            }
            else
                ErrorBorder.Visibility = Visibility.Collapsed;
        }

        private void LoadNavigator()
        {
            Navigator.ItemsSource = ExampleListings.GetExamples();
        }

        private async void RunCurrentClick(object sender, RoutedEventArgs e)
        {
            await RunCurrentExample();
        }

        private void ViewExportLogClick(object sender, RoutedEventArgs e)
        {
            var appDirectory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            Debug.Assert(appDirectory != null, "appDirectory != null");

            try
            {
                Process.Start(Path.Combine(appDirectory, "ExportLog.txt"));
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, Title, MessageBoxButton.OK, MessageBoxImage.Information);
            }
        }

        private void Navigator_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var example = e.NewValue as Example;
            var area = e.NewValue as ExampleArea;
            var code = "";
            var description = String.Empty;
            if (example != null)
            {
                Status.CurrentExample = example;
                Status.CurrentArea = ExampleListings.GetExamples()
                    .FirstOrDefault(a => a.Examples.Any(ex => ex.Name == Status.CurrentExample.Name));
                ExampleName.Text = Status.CurrentExample.Name;
                try
                {
                    var snippet = Status.CurrentExample.GetFullCSharpCode();
                    description = snippet.Description;
                    code = snippet.CleanCode(true, true);
                }
                catch (Exception exception)
                {
                    MessageBox.Show(exception.Message, Title, MessageBoxButton.OK, MessageBoxImage.Information);

                    return;
                }

                PlayButton.Visibility = Visibility.Visible;
                ExportExampleButton.Visibility = Visibility.Visible;
                ExportAreaButton.Visibility = Visibility.Collapsed;

                StepsGrid.ItemsSource = example.Steps;
                StepsGrid.SelectedItem = example.Steps.FirstOrDefault();
            }
            else if (area != null)
            {
                Status.CurrentArea = area;
                ExampleName.Text = area.Description;
                PlayButton.Visibility = Visibility.Collapsed;
                ExportExampleButton.Visibility = Visibility.Collapsed;
                ExportAreaButton.Visibility = Visibility.Visible;
                StepsGrid.ItemsSource = null;
            }
            else
            {
                ExampleName.Text = "";
                PlayButton.Visibility = Visibility.Collapsed;
                ExportExampleButton.Visibility = Visibility.Collapsed;
                ExportAreaButton.Visibility = Visibility.Collapsed;
                StepsGrid.ItemsSource = null;
            }
            //Update the code
            StatusDetailsCSharpFull.Text = code;
            StatusDetailsCSharpFull.TextArea.TextView.BackgroundRenderers.Clear();
            //Update the description
            _changingTabs = true;
            try
            {
                if (!String.IsNullOrEmpty(description))
                {
                    var nextTabIndex = (_lastTabIndex == 0 && StatusDetailsDescriptionTab.Visibility != Visibility.Visible) ? (int?) 0 : null;
                    StatusDetailsDescription.Text = description;
                    StatusDetailsDescriptionTab.Visibility = Visibility.Visible;
                    if (nextTabIndex != null)
                        StatusDetailsTabControl.SelectedIndex = nextTabIndex.Value;
                }
                else
                {
                    var nextTabIndex = (StatusDetailsDescriptionTab.Visibility == Visibility.Visible) ? (int?) 1 : null;
                    StatusDetailsDescriptionTab.Visibility = Visibility.Collapsed;
                    if (nextTabIndex != null)
                        StatusDetailsTabControl.SelectedIndex = nextTabIndex.Value;
                }
            }
            finally
            {
                _changingTabs = false;
            }
        }

        private async void Navigator_OnMouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            await RunCurrentExample();
        }

        private void StatusDetailsTabControl_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!_changingTabs)
                _lastTabIndex = StatusDetailsTabControl.SelectedIndex;
        }

        private async Task RunCurrentExample(CancellationToken cancellationToken = default(CancellationToken))
        {
            if (Status.CurrentExample == null)
                return;

            if (await IsRunning())
                return;

            var exampleToRun = Status.CurrentExample;

            using (new WaitCursor())
            {
                await exampleToRun.Run(null, cancellationToken);

                if (exampleToRun == Status.CurrentExample) //User may have navigated away
                {
                    StepsGrid.ItemsSource = exampleToRun.Steps;

                    if (exampleToRun.Steps != null && exampleToRun.Steps.Any())
                        StepsGrid.SelectedItem = Status.CurrentExample.Steps.First();
                }
            }
        }

        private Task<bool> IsRunning()
        {
            if (Status.RunningExample != null)
            {
                MessageBox.Show("Please wait for current example to complete.", Title, MessageBoxButton.OK, MessageBoxImage.Information);

                return Task.FromResult(true);
            }

            return Task.FromResult(false);
        }

        private void Navigator_Loaded(object sender, RoutedEventArgs e)
        {
            var item = (TreeViewItem)(Navigator.ItemContainerGenerator.ContainerFromItem(Navigator.Items.CurrentItem));

            if (item != null)
                item.IsSelected = true;
        }
    }
}
