﻿using System;
using Dss.Api.Examples.Internal;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.SubjectLists;
using DataScope.Select.Api.Search;

namespace Dss.Api.Examples.Search.Historical
{
    [Example("Historical Search")]
    public class HistoricalSearchExamples
    {
        protected SearchContext SearchContext { get; set; }
        protected ExtractionsContext ExtractionsContext { get; set; }
        protected const string InstrumentListPrefix = "_examples";

        public HistoricalSearchExamples()
        {
            SearchContext = ContextHelper.CreateSearchContext();
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
        }

        [EndToEndExample(typeof(SearchContext), typeof(HistoricalSearchRequest))]
        [Example("Search and add to Instrument List")]
        public void SearchAndAddToInstrumentList()
        {
            /*
             * This example demonstrates how to perform a historical search and then add the results to an instrument list. The instrument list is deleted at the end. */

            Status.Notify(SearchContext, null, "HistoricalSearch", MethodType.Operation, Publish.Primary);// Ignore STATUS stuff, used by the example application...

            DateTimeOffset startDate = new DateTime(2015, 11, 17, 0, 0, 0, DateTimeKind.Utc);
            DateTimeOffset endDate = new DateTime(2015, 11, 24, 0, 0, 0, DateTimeKind.Utc);

            // Setup the request...
            var request = new HistoricalSearchRequest
            {
                Identifier = "TRI.N",
                IdentifierType = IdentifierType.Ric,
                Range = new DateTimeRange { Start = startDate, End = endDate }
            };

            // Returns the results of the search...
            var results = SearchContext.HistoricalSearch(request);

            // Output...
            foreach (var historicalSearchResult in results)
                Status.WriteLine(historicalSearchResult.Identifier + " " + historicalSearchResult.Description);

            Status.EndNotify(SearchContext);// Ignore STATUS stuff, used by the example application...

            Status.Notify(ExtractionsContext, "InstrumentList", "Create", MethodType.Operation, Publish.Secondary);// Ignore STATUS stuff, used by the example application...

            // Define the Instrument List...
            var instrumentList = new InstrumentList()
            {
                ListId = "",
                Name = InstrumentListPrefix + Guid.NewGuid() + "_HistoricalSearch",
            };

            // Create the instrument list...
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);

            Status.EndNotify(ExtractionsContext);// Ignore STATUS stuff, used by the example application...

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers", MethodType.Operation, Publish.Secondary);// Ignore STATUS stuff, used by the example application...

            // Append the results of the search to the instrument list...
            var appendResult = ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, results, true);

            Status.EndNotify(ExtractionsContext);// Ignore STATUS stuff, used by the example application...

            Status.Notify(ExtractionsContext, "InstrumentList", "Delete", MethodType.Operation, Publish.Secondary);// Ignore STATUS stuff, used by the example application...

            // Delete the instrument list...
            ExtractionsContext.InstrumentListOperations.Delete(instrumentList);

            Status.EndNotify(ExtractionsContext);// Ignore STATUS stuff, used by the example application...
        }

        [EndToEndExample(typeof(SearchContext), typeof(HistoricalBulkSearchRequest))]
        [Example("Search list of identifiers")]
        public void SearchListOfIdentifiers()
        {
            /*
             * This example demonstrates how to perform a historical search with multiple identifiers. */

            Status.Notify(SearchContext, null, "HistoricalBulkSearch", MethodType.Operation, Publish.Primary);// Ignore STATUS stuff, used by the example application...

            var searchRequest = new HistoricalBulkSearchRequest() {
                Range = new DateTimeRange() {
                    Start = new DateTimeOffset(2015, 11, 17, 0, 0, 0, TimeSpan.Zero),
                    End = new DateTimeOffset(2015, 11, 24, 0, 0, 0, TimeSpan.Zero)
                }
            };

            searchRequest.Identifiers.Add(new HistoricalBulkSearchIdentifier() {
                Identifier = "TRI.N",   // Thomson Reuters
                IdentifierType = IdentifierType.Ric,
                Exchange = "",
                Currency = "",
                LineNumber = "0",
            });

            searchRequest.Identifiers.Add(new HistoricalBulkSearchIdentifier() {
                Identifier = "CA8849031056",   // Thomson Reuters
                IdentifierType = IdentifierType.Isin,
                Exchange = "",
                Currency = "",
                LineNumber = "1",
            });

            var result = SearchContext.HistoricalBulkSearch(searchRequest);

            // Output...
            foreach ( var historicalSearchResult in result.Results )
                Status.WriteLine(historicalSearchResult.Identifier + " " + historicalSearchResult.Description);

            Status.EndNotify(SearchContext);// Ignore STATUS stuff, used by the example application...
        }

    }
}
