﻿using System;
using System.IO;
using System.Linq;
using System.Threading;
using Dss.Api.Examples.Internal;
using DataScope.Select.Api.StandardExtractions;

namespace Dss.Api.Examples.StandardExtractions
{
    [Example("Standard Extractions")]
    public class StandardExtractionsExamples
    {
        protected StandardExtractionsContext StandardExtractionsContext { get; set; }

        public StandardExtractionsExamples()
        {
            StandardExtractionsContext = ContextHelper.CreateStandardExtractionsContext();
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(Subscription))]
        [Example("Get all Subscriptions")]
        public void GetSubscriptions()
        {
            /*
             * This example demonstrates how to retrieve all available subscriptions.
             * 
             * User entitlements are not taken into account. */

            Status.Notify(StandardExtractionsContext, "Subscription", "GetAll", MethodType.Operation, Publish.Primary);

            //Returns the list of all available subscriptions.
            var subscriptions = StandardExtractionsContext.SubscriptionOperations.GetAll();

            //Output
            foreach (var subscription in subscriptions)
                Status.WriteLine(subscription.SubscriptionId + " " + subscription.Name);

            Status.EndNotify(StandardExtractionsContext);
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(Package))]
        [Example("Get all Packages")]
        public void GetPackages()
        {
            /*
             * This example demonstrates how to retrieve all available packages.
             * 
             * User entitlements are not taken into account. */

            Status.Notify(StandardExtractionsContext, "Package", "GetAll", MethodType.Operation, Publish.Primary);

            //Returns the list of all available packages.
            var packages = StandardExtractionsContext.PackageOperations.GetAll();

            //Output
            foreach (var package in packages)
                Status.WriteLine(package.PackageId + " " + package.Name);

            Status.EndNotify(StandardExtractionsContext);
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(Package))]
        [Example("Get all Packages by Subscription")]
        public void GetPackagesForSubscription()
        {
            /*
             * This example demonstrates how to retrieve all available packages for a specific subscription. Remove the "Take(x)" to process all of the results returned.
             * 
             * User entitlements are not taken into account. */

            Status.Notify(StandardExtractionsContext, "Subscription", "GetAll");

            //Returns the list of all available subscriptions.
            var subscriptions = StandardExtractionsContext.SubscriptionOperations.GetAll();

            Status.EndNotify(StandardExtractionsContext);

            foreach (var subscription in subscriptions.Take(1))
            {
                Status.Notify(StandardExtractionsContext, "Package", "GetPackagesBySubscriptionId", MethodType.Operation, Publish.Primary);

                //Returns the packages for the requested subscription.
                var packages = StandardExtractionsContext.PackageOperations.GetPackagesBySubscriptionId(subscription.SubscriptionId);

                //Output
                foreach (var package in packages)
                    Status.WriteLine(package.PackageId + " " + package.Name);

                Status.EndNotify(StandardExtractionsContext);
            }
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(UserPackage))]
        [Example("Get my Packages")]
        public void GetMyPackages()
        {
            /*
             * This example demonstrates how to retrieve all available packages for a user. Remove the "Take(x)" to process all of the results returned.
             * 
             * User entitlements are taken into account. */

            Status.Notify(StandardExtractionsContext, "UserPackage", "GetAll", MethodType.Operation, Publish.Primary);

            //Returns the list of packages for which this user has permissions.
            var userPackages = StandardExtractionsContext.UserPackageOperations.GetAll();

            //Output
            foreach (var userPackage in userPackages.Take(10))
            {
                Status.WriteLine("{0} ({1})", userPackage.PackageName, userPackage.SubscriptionName);
            }

            Status.EndNotify(StandardExtractionsContext);
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(UserPackageDelivery))]
        [Example("Get my Package Deliveries by Package")]
        public void GetMyPackageDeliveriesByPackage()
        {
            /*
             * This example demonstrates how to retrieve all available package deliveries for a given package. Remove the "Take(x)" to process all of the resuls returned. 
             * 
             * User entitlements are taken into account. */

            Status.Notify(StandardExtractionsContext, "UserPackage", "GetAll");

            //Returns the list of packages for which this user has permissions.
            var userPackages = StandardExtractionsContext.UserPackageOperations.GetAll();

            Status.EndNotify(StandardExtractionsContext);

            //Output
            Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetUserPackageDeliveriesByPackageId", MethodType.Operation, Publish.Primary);
            foreach (var userPackage in userPackages.Take(1))
            {
                //Returns the users package deliveries for the requested package.
                var userPackageDeliveries = StandardExtractionsContext.UserPackageDeliveryOperations.GetUserPackageDeliveriesByPackageId(userPackage.PackageId);

                //Output
                foreach (var userPackageDelivery in userPackageDeliveries)
                {
                    Status.WriteLine("{0} ({1} bytes)", userPackageDelivery.Name, userPackageDelivery.FileSizeBytes);
                }
            }

            Status.EndNotify(StandardExtractionsContext);
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(UserPackageDelivery))]
        [Example("Get my Package Deliveries by Date Range")]
        public void GetMyPackageDeliveriesByDateRange()
        {
            /*
             * This example demonstrates how to retrieve all available package deliveries for a given subscription and date range. Remove the "Take(x)" to process all of the results returned.
             * 
             * User entitlements are taken into account. */

            Status.Notify(StandardExtractionsContext, "SubscriptionOperations", "GetAll");

            //Returns the list of all available subscriptions.
            var subscriptions = StandardExtractionsContext.SubscriptionOperations.GetAll();

            Status.EndNotify(StandardExtractionsContext);

            //Output
            Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetUserPackageDeliveriesByDateRange", MethodType.Operation, Publish.Primary);
            var fromDate = DateTime.UtcNow.AddDays(-1);
            var toDate = DateTime.UtcNow.AddDays(1);
            foreach (var subscription in subscriptions.Take(1))
            {
                //Returns the list of package deliveries for which this user has permissions for a specific subscription and a range of dates.
                var userPackageDeliveries = StandardExtractionsContext.UserPackageDeliveryOperations.GetUserPackageDeliveriesByDateRange(subscription.SubscriptionId, fromDate, toDate);

                //Output
                foreach (var userPackageDelivery in userPackageDeliveries.Take(10))
                {
                    Status.WriteLine("{0} ({1} bytes)", userPackageDelivery.Name, userPackageDelivery.FileSizeBytes);
                }
            }

            Status.EndNotify(StandardExtractionsContext);
        }

        [EndToEndExampleAttribute(typeof(StandardExtractionsContext), typeof(UserPackageDelivery))]
        [EndToEndExampleAttribute(typeof(StandardExtractionsContext))]
        [Example("Download my Package Deliveries")]
        public void DownloadMyPackageDelivery()
        {
            /*
             * This example demonstrates how to download a specific user package delivery. Remove the "Take(x)" to process all of the results returned.
             * 
             * User entitlements are taken into account. */

            Status.Notify(StandardExtractionsContext, "UserPackage", "GetAll");

            //Returns the list of packages for which this user has permissions.
            var userPackages = StandardExtractionsContext.UserPackageOperations.GetAll();

            Status.EndNotify(StandardExtractionsContext);

            //Output
            foreach (var userPackage in userPackages.Take(1))
            {
                Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetUserPackageDeliveriesByPackageId");

                //Returns the users package deliveries for the requested package.
                var userPackageDeliveries = StandardExtractionsContext.UserPackageDeliveryOperations.GetUserPackageDeliveriesByPackageId(userPackage.PackageId);

                Status.EndNotify(StandardExtractionsContext);

                //Output
                foreach (var userPackageDelivery in userPackageDeliveries.Take(1))
                {
                    Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetReadStream", MethodType.Operation, Publish.Primary);

                    Status.WriteLine("Downloading {0} ({1} bytes)", userPackageDelivery.Name, userPackageDelivery.FileSizeBytes);

                    using (var response = StandardExtractionsContext.UserPackageDeliveryOperations.GetReadStream(userPackageDelivery.PackageDeliveryId))
                    {
                        using (var fileStream = File.Create("..\\" + userPackageDelivery.Name))
                            response.Stream.CopyToAsync(fileStream).Wait();
                    }

                    Status.EndNotify(StandardExtractionsContext);
                }
            }
        }

        #region Tick History

        [EndToEndExample(typeof(StandardExtractionsContext))]
        [Example("Get my Venue by day data")]
        public void GetMyVenueData()
        {
            /*
             * This example demonstrates the steps necessary to retrieve my specific TickHistory Venue data.  This process utilizes 3 key features of the Rest API to get the job done. 
             * 
             * 1) Paging
             * 2) Change Tracking
             * 3) Download
             * 
             * Please refer to Key Mechanisms and other Standard Extractions examples for additional information.
             */

            // Set the max page size to 10 records per page for the purposes of this example...
            // The default is 250...
            StandardExtractionsContext.Preferences.MaxPageSize = 10;

            // Enable ChangeTracking...
            StandardExtractionsContext.Preferences.TrackChanges = true;

            Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetAll");// Ignore STATUS stuff, used by the example application...

            // Returns the list of package deliveries based on my entitlements...
            // Because ChangeTracking was enabled, the pagedResults will have a "DeltaLink" value...
            // That value needs to be saved and used the next time you ask for your data via "GetAll(DeltaLink from previous call, not available if 1st call)"...
            var pagedResults = StandardExtractionsContext.UserPackageDeliveryOperations.GetAll();

            // Save the DELTA LINK for use later...
            var savedDeltaLink = pagedResults.DeltaLink;

            Status.EndNotify(StandardExtractionsContext);

            // Process the results until no more pages...
            var loopCounter = 0; // Used to prematurely stop the example...
            while (loopCounter < 1 && pagedResults.HasNextPage)
            {
                loopCounter++;

                // Remove the "Take" to get all results...
                foreach (var packageDelivery in pagedResults.Take(1))
                {
                    Status.Notify(StandardExtractionsContext, "PackageDelivery", "GetReadStream", MethodType.Operation, Publish.Primary);
                    Status.WriteLine("Downloading {0} ({1} bytes)", packageDelivery.Name, packageDelivery.FileSizeBytes);

                    // Get the physical stream using the API...
                    using (var response = StandardExtractionsContext.UserPackageDeliveryOperations.GetReadStream(packageDelivery.PackageDeliveryId))
                    {
                        // Copies the stream to your local file system...
                        // Change the path in the File.Create to your desired location...
                        using (var fileStream = File.Create("..\\" + packageDelivery.Name))
                            response.Stream.CopyToAsync(fileStream).Wait();
                    }

                    Status.EndNotify(StandardExtractionsContext);
                }

                Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetAll");

                // Get the next page of results (if any) passing in the NextLink...
                pagedResults = StandardExtractionsContext.UserPackageDeliveryOperations.GetAll(pagedResults.NextLink);

                Status.EndNotify(StandardExtractionsContext);
            }

            // Let some time pass and call GetAll again using the persisted DeltaLink
            // There may not be any NEW items. It just depends on the amount of time that has passed
            Thread.Sleep(2000);

            // These results will be anything that is NEW since the DELTALINK was generated...
            Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetAll");
            pagedResults = StandardExtractionsContext.UserPackageDeliveryOperations.GetAll(savedDeltaLink);
            Status.EndNotify(StandardExtractionsContext);

            // Process the results...
            loopCounter = 0; // Used to prematurely stop the example...
            while (loopCounter < 2 && pagedResults.HasNextPage)
            {
                loopCounter++;

                // Remove the "Take" to get all results...
                foreach (var packageDelivery in pagedResults.Take(1))
                {
                    Status.Notify(StandardExtractionsContext, "PackageDelivery", "GetReadStream", MethodType.Operation, Publish.Primary);
                    Status.WriteLine("Downloading {0} ({1} bytes)", packageDelivery.Name, packageDelivery.FileSizeBytes);

                    // Get the physical stream using the API...
                    using (var response = StandardExtractionsContext.UserPackageDeliveryOperations.GetReadStream(packageDelivery.PackageDeliveryId))
                    {
                        // Copies the stream to your local file system...
                        // Change the path in the File.Create to your desired location...
                        using (var fileStream = File.Create("..\\" + packageDelivery.Name))
                            response.Stream.CopyToAsync(fileStream).Wait();
                    }

                    Status.EndNotify(StandardExtractionsContext);
                }

                Status.Notify(StandardExtractionsContext, "UserPackageDelivery", "GetAll");

                // Get the next page of results (if any) passing in the NextLink...
                pagedResults = StandardExtractionsContext.UserPackageDeliveryOperations.GetAll(pagedResults.NextLink);

                Status.EndNotify(StandardExtractionsContext);
            }
        }

        #endregion
    }
}
